#' Convert currently available data into a summary array
#' 
#' The function converts currently available data into an array of sufficient statistics.
#' 
#' @param Data.matrix
#' Data.matrix
#' @param nr.agents
#' The number of experimental plus control agents in the trial.
#' @param time.current
#' Current process time of the trial.
#' @param time.early
#' The time between from beginning of treatment and availablility of the first available endpoint.
#' @param time.final
#' The time between from beginning of treatment and availablility of the second available endpoint.
#'
#' @return
#' The function returns an array \code{Events} of dimension  \code{2 * nr.agents * 4}, where:
#' 
#' (a) \code{Events[,,1]} is a matrix \code{2 * nr.agents} of observed outcomes and observed responders by each agent for the 1. enpoint.
#' 
#' (b) \code{Events[,,2]} is a matrix \code{2 * nr.agents} of observed outcomes and observed responders by each agent for the 2. enpoint among all responders at the first endpoint. 
#' 
#' (c) \code{Events[,,3]} is a matrix \code{2 * nr.agents} of observed outcomes and observed responders by each agent for the 2. enpoint among all non-responders at the first endpoint. 
#' 
#' (d) \code{Events[,,4]} is a matrix \code{2 * nr.agents} of observed toxicity-outcomes and observed toxicity-event.
#'   
#' @examples
#' set.seed(121)
#' ### generate one dataset of trial outcome data containing 30 patients
#' Data = generate.data(nr.datasets           = 1, 
#'                      n                     = 30, 
#'                      arrival.rate          = 1, 
#'                      p.interim.1           = rep(0.4, 4), 
#'                      p.interim.2.success   = rep(0.9, 4), 
#'                      p.interim.2.failure   = rep(0.3, 4),
#'                      p.primary.success     = rep(0.9, 4),
#'                      p.primary.failure     = rep(0.3, 4),
#'                      p.toxicity            = rep(0.2, 4))
#' 
#' ### randomize all 30 patients at random to a treatment agent                                            
#' Data[[1]][,2] = sample(1:4, 30, TRUE)
#' 
#' ### retrive first and second outcome by agents 
#' get.data.matrix(
#' Data.matrix  = Data[[1]], 
#' nr.agents    = 4,   ## 4 agents
#' time.current = 25,  ## trial open for 25 weeks 
#' time.early   = 4,   ## 4 week 1. endpoint
#' time.final   = 8)   ## 8 week 1. endpoint
#'  
#' @export


get.data.matrix = function(Data.matrix, nr.agents, time.current, time.early, time.final){
 
 Events                   = array(dim=c(2,nr.agents, 4), dimnames=list(c("Total","Success"), 
                                                                       paste0("agent_", 1:nr.agents),
                                                                       c("surrogate", "final.given.sucess", "final.given.failure", "Toxicity") ))
 
 is.agent                 = sapply(1:nr.agents, function(a) Data.matrix[,2] ==a )
 is.early.endpt.available = time.current - Data.matrix[,1] >= time.early
 is.final.endpt.available = time.current - Data.matrix[,1] >= time.final
 is.early.responder       = sapply(1:nr.agents, function(a) Data.matrix[,2+a] == 1)
 
 ## surrogate events
 Events[1,,1]             = sapply(1:nr.agents, function(a) sum(            is.agent[,a] & is.early.endpt.available)       )
 Events[2,,1]             = sapply(1:nr.agents, function(a) sum(Data.matrix[is.agent[,a] & is.early.endpt.available, 2+a]) )
 
 ## final events given surrogate sucsess  
 Events[1,,2]             = sapply(1:nr.agents, function(a) sum(            is.agent[,a] & is.final.endpt.available & is.early.responder[,a])     )
 Events[2,,2]             = sapply(1:nr.agents, function(a) sum(Data.matrix[is.agent[,a] & is.final.endpt.available & is.early.responder[,a], a+nr.agents+2]) )
 
 ## final events given surrogate failure  
 Events[1,,3]             = sapply(1:nr.agents, function(a) sum(            is.agent[,a] & is.final.endpt.available & !is.early.responder[,a])     )
 Events[2,,3]             = sapply(1:nr.agents, function(a) sum(Data.matrix[is.agent[,a] & is.final.endpt.available & !is.early.responder[,a], a+nr.agents+2]) )
 
 ## toxicity events 
 Events[1,,4]             = Events[1,,1]
 Events[2,,4]             = sapply(1:nr.agents, function(a) sum(Data.matrix[is.agent[,a] & is.final.endpt.available,  a+3*nr.agents+2]) )
 
return(Events) }


