#' P-value for testing of treatment efficacy for the adaptive trial design
#'
#' The function computes a Bootstrap p-value for the Null hypothesis, 
#' \eqn{H_a: p_a \leq p_0-non.inf.delta},
#' that a given experimental agent
#' has a lower response-to-treatment probability  \code{p_a}  
#' for the primary endpoint than the active/historical response rate  \code{p_0} by at least \code{non.inf.delta}.
#' The test is computed independently of each treatment regimen \code{a}.
#' Note that this is a computational intensive procedure which may take up 20 minutes to compute.
#' 
#' @param Trial
#' Outcome  data for the adaptive trial Trial
#' 
#' @param test.arm Which regemen should be tested
#' 
#' @param active.control
#' If \code{active.control = TRUE}, 
#' trial with active control, 
#' otherwise, with \code{active.control = FALSE}, a trial with historical control with response rate \code{p_0=HRR} is used for comparison.
#' 
#' @param non.inf.delta
#' The non-inferiority margin.
#' 
#' @param arrival.rate
#' Expected or estimated accrual rate of the trial. 
#' 
#' @param Prior11 
#' A vector of two positive real numbers of Beta-prior parameters of a response at the final outcome given a positive response at the surrogate outcome.
#'   
#' @param Prior10 
#' A vector of two positive real numbers of Beta-prior parameters of a response at the final outcome given a negative response at the surrogate outcome.  
#' 
#' @param t_SEP_OK 
#' Average time in weeks until the surrogate endpoint will be available.  
#' 
#' @param t_EP_OK 
#' Average time in weeks until the final endpoint will be available.
#' 
#' @param TOX.rate  
#' Toxicity threshold \code{TOX.rate}, i.e. tha maximal acepable frequencey of toxcicity related events.  
#' 
#' @param TOX.threshold 
#' Stop an agent for toxicity if the posterior probability of the observed toxiticty rate greater than the threshold \code{TOX.rate} falls above the value \code{TOX.threshold}. 
#' 
#' @param PTE.threshold 
#' Stop an agent for futility if the posterior probability of a positive treatment effect drops below the value \code{PTE.threshold}. 
#' 
#' @param HRR 
#' Historical response probability for the primary endpoint.
#' 
#' @param Delta
#' Inferiority Boundary.
#' 
#' @param a 
#' Scale parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param b 
#' Exponent parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param c 
#' Exponent parameter of the power function used to amplify the randomization to the control nr.arms.
#'
#' @param nr.datasets 
#' Number of bootstrap simulation
#'
#' @param acc.min 
#' Required minimum number of patients randomized to each arm.
#' 
#' @return 
#' Returns a single p-value.
#' @examples   
#' ## generate trial-outcome data
#' Data = generate.data(nr.datasets           = 1, 
#'                      n                     = 600, 
#'                      arrival.rate          = 1, 
#'                      p.interim.1           = rep(0.4, 4), 
#'                      p.interim.2.success   = rep(0.9, 4), 
#'                      p.interim.2.failure   = rep(0.3, 4),
#'                      p.primary.success     = rep(0.9, 4),
#'                      p.primary.failure     = rep(0.3, 4),
#'                      p.toxicity            = rep(0.2, 4))
#' 
#' # simulat an adaptive trial with active control (agent 1)
#' Simulate_Trial= sim.trial( 
#'  X                = Data[[1]],
#'  active.control   = TRUE,
#'  Prior11          = c(.1*0.8,.1*.2), ## prior probability
#'  Prior10          = c(.1*0.3,.1*.7), ## prior probability
#'  t_SEP_OK         = 4,               ## surrogate endpoint available after 4 week 
#'  t_EP_OK          = 14,              ## primary endpoint available after 4 week
#'  HRR              = NULL,            ## No historical response rate
#'  Delta            = 0,               ## no non-inferiority margin
#'  a                = (1/250)^5,       ## 1. randomization parameter
#'  b                = 5,               ## 2. randomization parameter
#'  c                = .25,             ## 3. randomization parameter
#'  TOX.rate         = .2, 
#'  TOX.threshold    = .8,
#'  PTE.threshold    = 0.1, 
#'  acc.min          = 10)              ## require at least 10 patients per regiment
#' 
#' ## get the p-value for the 1. experimental agents (2.agent in the trial)                                              
#'P.value.Bootstrap(Trial=Simulate_Trial,
#'   active.control  = TRUE,
#'   test.arm=2,
#'   Prior11         = c(.1*0.8,.1*.2), ## prior probability
#'   Prior10         = c(.1*0.3,.1*.7), ## prior probability
#'   t_SEP_OK        = 4,               ## 1 surrogate endpoint after 14 week 
#'   t_EP_OK         = 14,              ## 2 surrogate endpoint  after 42 week
#'   HRR             = NULL,
#'   Delta           = 0,               ## no non-inferiority margin
#'   a               = (1/250)^5,       ## 1. rand parameter
#'   b               = 5,               ## 2. rand parameter
#'   c               = .25,             ## 3. rand parameter
#'   TOX.rate        = .2, 
#'   TOX.threshold   = .8,
#'   PTE.threshold   = 0.1, 
#'   acc.min         = 10, 
#'   nr.datasets     = 50)                
#' @export





P.value.Bootstrap=function(Trial,
                         active.control = TRUE, 
                         test.arm,
                         non.inf.delta = 0.12,
                         arrival.rate  = 4,
                         Prior10       = c(1,1), 
                         Prior11       = c(1,1),
                         t_SEP_OK      = 14,     
                         t_EP_OK       = 34, 
                         HRR           = NULL,
                         Delta         = 0,
                         TOX.rate      = 1, 
                         TOX.threshold = 1,
                         PTE.threshold = 0, 
                         a             = 0.004,   
                         b             = 1.3, 
                         c             = 2,       
                         acc.min       = 0,
                         nr.datasets   = 500)

{
 
 Sample.Size           = table(Trial$X[,2])
 
 
 ###### MLE's of response probabilities ############################################################
 ### MLE of response probability at fist interim outcome
 Sample.interim.1      = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a,  2 + a]) )
 ### MLE of response probability at second interim outcome given response at first interim outcome
 Sample.interim.2.succ = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a & Trial$X[,2+a] == 1,  2 + 1*nrow(Sample.Size) +a]) )
 ### MLE of response probability at second interim outcome given failure at first interim outcome
 Sample.interim.2.fail = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a & Trial$X[,2+a] == 0,  2 + 1*nrow(Sample.Size) +a]) )
 ### MLE of response probability at second interim outcome
 Sample.interim        = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a,  2 + 1*nrow(Sample.Size) +a]) )
 
 Sample.primary.succ   = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a & Trial$X[,2+1*nrow(Sample.Size) +a] == 1,  2 + 2*nrow(Sample.Size) +a]) )
 Sample.primary        = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a,  2 + 2*nrow(Sample.Size) +a]) )
 
 
 ###### MLE under NULL of primary response probabilities ############################################################
 #primary.NULL          = Sample.primary[1]*(Sample.Size[1]/(Sample.Size[1]+Sample.Size[test.arm]))+Sample.primary[test.arm]*(1-Sample.Size[1]/(Sample.Size[1]+Sample.Size[test.arm]))
 X.0                   = Sample.Size[1]*Sample.primary[1]
 X.a                   = Sample.Size[test.arm]*Sample.primary[test.arm]
 primary.NULL.Num      = non.inf.delta*(1/(Sample.Size[test.arm]-X.a)+1/X.a) +1/(Sample.Size[test.arm]-X.a)+1/(Sample.Size[1]-X.0)
 primary.NULL.Denum    = 1/X.a + 1/X.0 +1/(Sample.Size[test.arm]-X.a)  +1/(Sample.Size[1]-X.0)                                      
 primary.NULL          = primary.NULL.Num/primary.NULL.Denum
 Sample.primary[1]     = primary.NULL
 Sample.primary[test.arm] = primary.NULL-non.inf.delta

 
 #Sample.primary.fail = (Sample.primary-Sample.primary.succ*Sample.interim)/(1-Sample.interim) 
 Sample.primary.succ = 1-0.05/Sample.primary
 Tox.Resp            = sapply(1:nrow(Sample.Size), function(a) mean(Trial$X[Trial$X[,2] == a,  2 + 3*nrow(Sample.Size) +a]) )
 
 Data.Boot=generate.data(nr.datasets, n=nrow(Trial$X) ,
                    arrival.rate=arrival.rate,
                    p.interim.1=Sample.interim.1,
                    p.interim.2.success=Sample.interim.2.succ,
                    p.interim.2.failure=Sample.interim.2.fail,
                    p.primary.success=Sample.primary.succ,
                    p.primary.failure=rep(0,nrow(Sample.Size)),
                    p.toxicity=Tox.Resp)

 Boot.gen= lapply(1:length(Data.Boot), function(i) sim.trial(
                       X=Data.Boot[[i]], 
                       active.control = active.control,
                       Prior10  = Prior10, 
                       Prior11 = c(1,1),
                       t_SEP_OK = t_SEP_OK,     
                       t_EP_OK = t_EP_OK, 
                       HRR      = HRR,
                       Delta         = Delta,
                       TOX.rate      = TOX.rate, 
                       TOX.threshold = TOX.threshold,
                       PTE.threshold = PTE.threshold, 
                       a        = a,   b      = b, 
                       c        = c,       acc.min=acc.min))
 
 
   NULL_DIST = sapply(1:length(Boot.gen), function(i){
    
    if(Boot.gen[[i]]$STATUS[1,test.arm]==0){
     N   = table(Boot.gen[[i]]$X[,2])[c(1,test.arm)]
     P.0 = mean(Boot.gen[[i]]$X[Boot.gen[[i]]$X[,2] == 1,         2 + 2*nrow(Sample.Size) +1]) 
     P.a = mean(Boot.gen[[i]]$X[Boot.gen[[i]]$X[,2] == test.arm,  2 + 2*nrow(Sample.Size) +test.arm])
     Z   = (P.a-P.0-non.inf.delta) /  sqrt( P.0 * (1-P.0)/N[1] + P.a * (1-P.a)/N[test.arm])
           return(Z)
    }else  return(NA)

    })

 NULL_DIST = NULL_DIST[!is.na(NULL_DIST)]          
 RR.mle    =  sapply(c(1, test.arm), function(a) mean(Trial$X[Trial$X[,2] == a,  2 + 2*nrow(Sample.Size) +a]) )
 stat_0    = (RR.mle[2]-RR.mle[1]-non.inf.delta)/sqrt(RR.mle[1]*(1-RR.mle[1])/Sample.Size[1]+RR.mle[2]*(1-RR.mle[2])/Sample.Size[test.arm]) 
 PVALUE    = mean(NULL_DIST>=stat_0)

 return(PVALUE)
}


