#' Generate random numbers of treatment assignment
#' 
#' The function computes the vector of Bayesian adaptive randomizatio probabilities \eqn{p=(p_1, \cdots, p_A)} based on the vector of posterior probabilities of a positive treatment effect.
#' And generates iid random treatment assignments from p such that each treatment assignment equals agents a with probability \eqn{p_a}.
#' 
#' 
#' @param Pr.PTE 
#' A cector of posterior probabilities  of a positive treatment effect (PTE) computed with \code{Posterior.Sup}.
#' 
#' @param Observed.events
#' The total number of observed second interim events in the trial.
#' 
#' @param accrual
#' The total number of treatment assignments in the trial. 
#' 
#' @param acc.min
#'  Required minimum number of patients randomized to each arm.
#'
#' @param a 
#' Scale parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param b 
#' Exponent parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param c 
#' Exponent parameter of the power function used to amplify the randomization to the control nr.arms.
#' 
#' @param STATUS
#' Status matrix of dimension \eqn{2 \times } number of agents in the trial. 
#' Where the \code{STATUS[1,a]==0} indicates that agent \code{a} is still active and \code{STATUS[1,a]==1,2} indicate that agent \code{a} was dropted for futility or toxicity at time \code{STATUS[2,a]}. 
#'
#' @param n.draws
#' Number of random numbers, which should be generated. 
#' 
#' 
#' @param active.control
#' If \code{active.control = TRUE}, randomiza with active control, otherwise randomiza without active control.
#' 
#' @return
#' A vector of \code{n.draws} random numbers.
#' 
#' 
#' 
#' @examples
#' ## generate toy data
#' Events.fin.cond.succsess = rbind(c(12, 10, 14, 10), c( 6,  3,  9,  5))
#' Events.fin.cond.failure  = rbind(c(12, 10, 14, 10), c( 3,  1,  4,  2))
#' Events.sur               = rbind(c(13, 10, 10, 20), c( 2,  4,  5,  8))
#' nr.agnets                = 4
#'
#' Names     = list( c("Total","Success"), 
#'                   paste0("agent_", 1:nr.agnets), 
#'                  c("surrogate", "final.given.sucess", "final.given.failure"))
#'                  
#' Events     = array(0, dim=c(2, nr.agnets, 3), dimname=Names)
#' Events[,,1]=Events.sur
#' Events[,,2]=Events.fin.cond.succsess
#' Events[,,3]=Events.fin.cond.failure
#' 
#' ## show all data 
#' Events
#' 
#' ## treat first agent as standard of care 
#' ## and compare agent 2-4 to the active standard of care 
#' P.PTE = Posterior.Sup(  Events, 
#'                         HRR                     = NULL,
#'                         active.control          = TRUE,
#'                         Prior.fin.cond.succsess = c(1,1),
#'                         Prior.fin.cond.failure  = c(1,1))
#' 
#' ## draw 20 random treatment assignments                        
#' Rand.pr(Pr.PTE           = P.PTE, 
#'         n.draws          = 20,                 ## require 20 random numbers 
#'         Observed.events  = sum(Events[1,,-1]), 
#'         accrual          = rowSums(Events[1,,-1]), 
#'         acc.min          = 10,      
#'         a=1, b=0, c=0,   
#'         active.control   = TRUE,            ## agent 1 is active control
#'         STATUS           = matrix(0, 2, 3)) ## all agants are active
#'                          
#' @export                         


Rand.pr = function(Pr.PTE, n.draws=1, Observed.events, accrual, acc.min=0, a, b, c, STATUS, active.control=TRUE){
 
               Power  = a*Observed.events^b

if(active.control){
                p     = Pr.PTE^Power
                p.sum = sum(p) 
               
   if(p.sum>0){ p     = p/p.sum 
                p     = c(exp(c*( max(accrual[-1])-accrual[1] )) /length(Pr.PTE), p) * (STATUS[1,] ==0)
                p     = p/sum(p)
                
   }else        p     = c(1,p)      
                
   
 }else{         p     = Pr.PTE^Power * (STATUS[1,]==0)
                p.sum = sum(p)
   if(p.sum>0){ p     = p/p.sum  
   }else        p     = rep(1, length(Pr.PTE))/length(Pr.PTE)            
 }


 p = (p+1/10^4) * exp(2*pmax(acc.min-accrual, 0)) 
 p = p/sum(p)

return(sample(1:length(p), size=n.draws, replace=TRUE, prob=p ))}
