#' Generate arrival and outcome data
#' 
#' The function generates multiple datasets of patient arrival times, 
#' two surrogate enpoints,  the primary outcome, and 
#' the toxicity enpoint for each patient and each treatment agent.
#'  
#' 
#' @param nr.datasets 
#' Number of datasets to be generated.
#' 
#' @param n 
#' Total number of patients for each dataset in the trial. 
#' 
#' @param arrival.rate 
#' The expected average number of patients entering the trial within one week. 
#' 
#' @param p.interim.1 
#' A vector of response probabilities for the first interim outcome for each agent.
#' 
#' @param p.interim.2.success
#' A vector of response probabilities for the second interim outcome for each agent, 
#' given a favourable outcome for the first interim outcome.
#' 
#' @param p.interim.2.failure
#' A vector of response probabilities for the second interim outcome for each agent, 
#' given an unfavourable outcome for the first interim outcome.
#' 
#' @param p.primary.success
#' A vector of response probabilities for the primary outcome for each agent, 
#' given a favourable outcome for the second interim outcome.
#' 
#' @param p.primary.failure
#' A vector of response probabilities for the primary outcome for each agent, 
#' given an unfavourable outcome at the second interim outcome.
#' 
#' @param p.toxicity
#' Probability of a toxicity related event.
#' 
#' @return
#' The funcion returns a list of \code{nr.datasets} trial datasets. 
#' 
#' Each dataset (element of the list) is a matrix  of \code{n} rows, wherer each row represents a patient. 
#' For each patient the first two entries (columns) indicate the arrival time and treatment-with-agent variable. 
#' At generation of the outcome data the treatment-with-agent information is unknown, hence the 2. column contains only \code{NA}'s.
#' The remaining \code{3} to \code{2 + 3*} columns contain the first and second interim endpoint, the primary endpoint 
#' and the troxicity outcome for each treatment agent.  
#' 
#' 
#' @examples
#' Data = generate.data(
#'   nr.datasets           = 1,   ## only one dataset
#'   n                     = 10,  ## total sample size of 10 patient
#'   arrival.rate          = 1,      ## average of one arriving patient per week  
#'   p.interim.1           = rep(0.4, 4), 
#'   p.interim.2.success   = rep(0.9, 4), 
#'   p.interim.2.failure   = rep(0.3, 4),
#'   p.primary.success     = rep(0.9, 4),
#'   p.primary.failure     = rep(0.3, 4),
#'   p.toxicity            = rep(0.2, 4))
#'                      
#'                      
#' @export


generate.data = 
function(nr.datasets, 
         n, 
         arrival.rate, 
         p.interim.1, 
         p.interim.2.success,
         p.interim.2.failure,         
         p.primary.success,
         p.primary.failure, 
         p.toxicity){
 
 n.agents = length(p.interim.1)
 
 if(n.agents != length(p.primary.success) || 
    n.agents != length(p.primary.failure) || 
    n.agents != length(p.toxicity) )
  stop("probability vectors must be of the same length")

 mat.interim.2 = rbind(p.interim.2.failure, p.interim.2.success)
 mat.primary   = rbind(p.primary.failure,   p.primary.success) 
   
 Data          = replicate(n = nr.datasets,
                  expr = {
  arrival.vec       = 1+round(cumsum(rexp(n, arrival.rate)))
  Outcome.interim.1 = sapply(p.interim.1, function(p) rbinom(n, 1, p))
  Outcome.interim.2 = sapply(1:n.agents,  function(i) rbinom(n, 1, mat.interim.2[1+Outcome.interim.1[,i],i]))
  Outcome.finale    = sapply(1:n.agents,  function(i) rbinom(n, 1,   mat.primary[1+Outcome.interim.2[,i],i]))
  Outcome.foxicity  = sapply(p.toxicity,  function(p) rbinom(n, 1, p)) 
  
  Data              = cbind(arrival.vec, rep(NA, n), Outcome.interim.1, Outcome.interim.2, Outcome.finale, Outcome.foxicity)
  rownames(Data)    = paste0("patient_", 1:n)
  colnames(Data)    = c("arrival_time", "Rand_Agent", 
                         paste0("Sur1_A", 1:n.agents), 
                         paste0("Sur2_A", 1:n.agents),
                         paste0("Fin_A",  1:n.agents),
                         paste0("Tox_A",  1:n.agents) )
                       return(Data)},  
              simplify =  FALSE)
  
return(Data) }






