#' Simulate Response Adaptive Trial 
#' 
#' The function simulates one response adaptive trial with early enpoints and a Bayesian adaptive randomization model.
#' 
#'  
#' @param X
#' Outcome data generated with \code{generate.data}.
#' 
#' @param active.control
#' If \code{active.control = TRUE}, trial with active control, otherwsie trial with historical control of response rate \code{HRR}.
#' 
#' @param Prior11 
#' A vector of two positive real numbers of Beta-prior parameters of a response at the final outcome given a positive response at the surrogate outcome.
#'   
#' @param Prior10 
#' A vector of two positive real numbers of Beta-prior parameters of a response at the final outcome given a negative response at the surrogate outcome.  
#' 
#' @param t_SEP_OK 
#' Average time in weeks until the surrogate endpoint will be available.  
#' 
#' @param t_EP_OK 
#' Average time in weeks until the final endpoint will be available.
#' 
#' @param TOX.rate  
#' Toxicity threshold \code{TOX.rate}, i.e. tha maximal acepable frequencey of toxcicity related events.  
#' 
#' @param TOX.threshold 
#' Stop an agent for toxicity if the posterior probability of the observed toxiticty rate greater than the threshold \code{TOX.rate} falls above the value \code{TOX.threshold}. 
#' 
#' @param PTE.threshold 
#' Stop an agent for futility if the posterior probability of a positive treatment effect drops below the value \code{PTE.threshold}. 
#' 
#' @param HRR 
#' Historical response probability for the primary endpoint.
#' 
#' @param Delta
#' Inferiority Boundary.
#' 
#' @param a 
#' Scale parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param b 
#' Exponent parameter of the power function \eqn{h(n)=an^c} used to amplify the posterior in the computation of the randomization probabilities, where \code{n} is the current number of observed events.
#' 
#' @param c 
#' Exponent parameter of the power function used to amplify the randomization to the control nr.arms.
#'
#' @param acc.min 
#' Required minimum number of patients randomized to each arm.
#'  
#' @return 
#' Returns a matrix of trial data.
#'
#' @examples 
#' ### generate outcome data under treatment with each agents.
#' Data = generate.data(
#'    nr.datasets           = 2,    ## simulate 2 datasets     
#'    n                     = 100,  ## each with 100 patients
#'    arrival.rate          = 1,    ## mean accrual of 1 patients per week
#'    p.interim.1           = rep(0.4, 4), 
#'    p.interim.2.success   = rep(0.9, 4), 
#'    p.interim.2.failure   = rep(0.3, 4),
#'    p.primary.success     = rep(0.9, 4),
#'    p.primary.failure     = rep(0.3, 4),
#'    p.toxicity            = rep(0.2, 4))
#' 
#' ### simulate a trial 
#' sim.trial( 
#'  X                = Data[[1]],
#'  active.control   = TRUE,
#'  Prior11          = c(.1*0.8,.1*.2), ## prior probability
#'  Prior10          = c(.1*0.3,.1*.7), ## prior probability
#'  t_SEP_OK         = 4,               ## surrogate endpoint available after 4 week 
#'  t_EP_OK          = 14,              ## primary endpoint available after 4 week
#'  HRR              = 0.6,             ## historical response rate
#'  Delta            = 0.12,            ## 12% non-inferiority margin
#'  a                = (1/250)^5,       ## first randomization parameter
#'  b                = 5,               ## second randomization parameter
#'  c                = .25,             ## third randomization parameter
#'  acc.min          = 10)              ## require at least 10 patients per regiment
#' 
#' @export

sim.trial = function(X, active.control = FALSE,
                     Prior10  = c(1,1), 
                     Prior11 = c(1,1),
                     t_SEP_OK = 14,     
                     t_EP_OK = 34, 
                     HRR      = NULL,
                     Delta         = 0,
                     TOX.rate      = 1, 
                     TOX.threshold = 1,
                     PTE.threshold = 0, 
                     a        = 0.004,   b      = 1.3, 
                     c        = 2,       acc.min=0){
 
 
 nr.arms   = (ncol(X)-2)/4
 N         = nrow(X)
 accrual   = rep(0, nr.arms)
 acc.min   = acc.min
 STATUS    = matrix(0, 2, nr.arms, dimnames = list(c("Status","Time"), paste0("Agent", 1:nr.arms) )) 
 nr.looks  = 0
 
 
	for(t in 1:X[N,1]){            patients.at.week.t    = X[,1]==t
                                
  if(any(patients.at.week.t)){  patients.at.week.t    = which(patients.at.week.t)
                                nr.patients.at.t      = length(patients.at.week.t)
                                rand.patient          = rep(0, nr.patients.at.t)
                                Events                = get.data.matrix(Data.matrix=X, nr.agents=nr.arms, time.current=t, time.early=t_SEP_OK, time.final=t_EP_OK)
                                Posterior.of.PTE      = Posterior.Sup(Events, HRR=HRR, Delta=Delta, active.control=active.control, Prior.fin.cond.succsess=Prior11, Prior.fin.cond.failure=Prior10)
                                stop.now              = (sum(Events[1,,2:3])  / ((nr.looks+1)*100)) >= 1                                
                                
  if(stop.now){                 STATUS                = Stopping.rule(STATUS=STATUS, Event.Tox.mat=Events[,,4], Pr.PTE=Posterior.of.PTE, TOX.rate=TOX.rate, TOX.threshold=TOX.threshold,  PTE.threshold=PTE.threshold, active.control=active.control, Time=t)
                                nr.looks              = nr.looks + 1 }
                                
 ## all patients within week t                                                                
 for(i in 1:nr.patients.at.t)
 {  
  rand.patient[i]          = Rand.pr(Pr.PTE=Posterior.of.PTE, n.draws=1, Observed.events=sum(Events[1,,c(2,3)]), accrual=accrual, acc.min=acc.min, a=a, b=b, c=c, active.control=active.control, STATUS=STATUS)  
  accrual[rand.patient[i]] = accrual[rand.patient[i]] + 1           
 }
  X[patients.at.week.t,2]  = rand.patient
	}
	
	}
 
 
return(list(X=X, STATUS=STATUS)) } 
