## ------------------------------------------------------------------------
library(BayesAdaptSurrogate)
cbind(ls(2))

## ------------------------------------------------------------------------
## probability of response to treatment at the 1. preliminary outcome
p.interim.1 = c(.3, .45, .3, .3)   

## probability of response to treatment at the 2. preliminary outcome
### given a positive response to treatment at the 1. preliminary outcome
p.interim.2.success = c(.8, .9, .8, .8)
## given a negative response to treatment at the 1. preliminary outcome
p.interim.2.failure = c(.4, .35, .4, .4)

## probability of response to treatment at the primary outcome
### given a positive response to treatment at the 2. preliminary outcome
p.primary.success     = c(0.9, 0.95, 0.9, 0.9)
## given a negative response to treatment at the 2. preliminary outcome
p.primary.failure     = c(.3, .25, .2, .2)

## ------------------------------------------------------------------------
set.seed(1)
Outcome.Data = BayesAdaptSurrogate::generate.data(
  nr.datasets           = 10,      ## only one dataset
  n                     = 120,     ## total sample size of 10 patient
  arrival.rate          = 1,       ## average of one arriving patient per week
  p.interim.1           = p.interim.1,
  p.interim.2.success   = p.interim.2.success,
  p.interim.2.failure   = p.interim.2.failure,
  p.primary.success     = p.primary.success,
  p.primary.failure     = p.primary.failure,
  p.toxicity            = rep(0.2, 4))

## ------------------------------------------------------------------------
Outcome.Data[[1]][2:3,]

## ------------------------------------------------------------------------
## avarage response at the 1. surrogate outcome
rbind("TRUE"=p.interim.1, "sample"=round(colMeans(Outcome.Data[[1]] [,(2+1):(2+4)]), 2))

## ------------------------------------------------------------------------
Trials = apply(matrix(1:length(Outcome.Data)), 1, function(i) BayesAdaptSurrogate::sim.trial(
 X               = Outcome.Data[[i]],
 active.control   = TRUE,
 Prior11          = c(.1*0.8,.1*.2), ## prior probability
 Prior10          = c(.1*0.3,.1*.7), ## prior probability
 t_SEP_OK         = 4,               ## surrogate endpoint available after 4 week
 t_EP_OK          = 14,              ## primary endpoint available after 4 week
 HRR              = .6,              ## historical response rate
 Delta            = .12,             ## 12% non-inferiority margin
 TOX.rate         = .3,
 TOX.threshold    = .7, 
 PTE.threshold    = .6,
 a                = (1/60)^3,        ## first randomization parameter
 b                = 3,               ## second randomization parameter
 c                = .25,             ## third randomization parameter
 acc.min          = 10))             ## require at least 10 patients per regiment

## ------------------------------------------------------------------------
Outcome.Data[[1]][1:5, 1:5]
Trials[[1]]$X[1:5, 1:5]

## ------------------------------------------------------------------------
Trials[[1]]$STATUS

## ------------------------------------------------------------------------
(Events = get.data.matrix(Data.matrix  = Trials[[1]]$X, 
                          nr.agents    = 4,   ## number of agents
                          time.current = 100, ## current time of the trial
                          time.early   = 12,  ## time to observed 1.surrogate outcome 
                          time.final   = 40)) ## time to observed 2.surrogate outcome

## ------------------------------------------------------------------------
## treat first agent as standard of care
## and compare agent 2-4 to the active standard of care
(PTE1 = Posterior.Sup(Events          = Events,
              HRR                     = NULL,
              active.control          = TRUE,
              Prior.fin.cond.succsess = c(1,1),
              Prior.fin.cond.failure  = c(1,1)))

## ------------------------------------------------------------------------
## treat all agents (1 to 4) as experimental agents
## and compare to historical control
(PTE2 = Posterior.Sup(Events = Events,
              HRR                     = .4, ### historical response rate
              active.control          = FALSE,
              Prior.fin.cond.succsess = c(1,1),
              Prior.fin.cond.failure  = c(1,1)))

## ------------------------------------------------------------------------
STATUS   = matrix(0, 2, 4)
### with active control 
(STATUS1 = Stopping.rule(STATUS  = STATUS,
              Event.Tox.mat  = Events,
              Pr.PTE         = PTE1,
              TOX.rate       = .5,
              PTE.threshold  = .1,
              TOX.threshold  = .9,
              active.control = TRUE,
              Time           = 40))
### without active control
### with active control 
(STATUS2 = Stopping.rule(STATUS  = STATUS,
              Event.Tox.mat  = Events,
              Pr.PTE         = PTE2,
              TOX.rate       = .5,
              PTE.threshold  = .1,
              TOX.threshold  = .9,
              active.control = FALSE,
              Time           = 40))

## ------------------------------------------------------------------------
## patient accrual by agent at week 40
accrual.by.agent = sapply(1:4, function(a) sum(Trials[[1]]$X[,1] <= 40 & Trials[[1]]$X[,2]==a) )

Rand.pr(Pr.PTE           = PTE1,
        n.draws          = 20,                            ## require 20 random numbers
        Observed.events  = sum(Events[1,,-1]),            ## number of observed 2. surrogate outcomes 
        accrual          = accrual.by.agent,              ## patient accrual by agent at week 40
        acc.min          = 10,                            ## require minimum of 10 observed events per agent
        a                = (1/60)^3,                      ## 1. rand parameter
        b                = 3,                             ## 2. rand parameter
        c                = 1,                             ## 3. rand parameter
        active.control   = TRUE,     ## agent 1 is active control
        STATUS           = STATUS1)  ## all arms are active

## ------------------------------------------------------------------------
Rand.pr(Pr.PTE           = PTE2,
        n.draws          = 20,                            ## require 20 random numbers
        Observed.events  = sum(Events[1,,-1]),            ## number of observed 2. surrogate outcomes 
        accrual          = accrual.by.agent,              ## patient accrual by agent at week 40
        acc.min          = 10,                            ## require minimum of 10 observed events per agent
        a                = (1/60)^3,                      ## 1. rand parameter
        b                = 3,                             ## 2. rand parameter
        c                = NULL,                          ## 3. rand parameter
        active.control   = FALSE,    ## no active control
        STATUS           = STATUS2)  ## agents 1 to 3 are active

